{############################################################## KIM1FILE.INC }
{ Purpose...: KIM1 include file: File functions
{ DependsOn.: LIB.INC
{ Compiler..: Turbo-Pascal V3.0
{ Target....: Kaypro II
{ LastEdit..: 22.01.2023
{############################################################################}

{*************************************************************** definitions }
VAR m_iBufSize    : INTEGER;                     { size of allocated buffer  }
VAR m_iBufUsed    : INTEGER;                     { used bytes in buffer      }
VAR m_iBufPos     : INTEGER;                     { actual read/write position}
VAR m_iBufLine    : INTEGER;                     { actual line in buffer     }
VAR m_iBufLines   : INTEGER;                     { total lines in buffer     }
VAR m_iFileSize   : INTEGER;                     { filesize in binary mode   }
VAR m_pstBufStart : ^g_tStLine;                  { buffer start address      }
VAR m_pstBuf      : ^g_tStLine;                  { current buffer position   }
VAR m_pcBuf       : ^CHAR;                       { current CHAR buffer pos   }
VAR m_pbBuf       : ^BYTE;                       { buffer position binary    }
VAR m_oFileTxt    : TEXT;                        { text file to send/receive }

{************************************************************ implementation }
{========================================================== File_CloseFile() }
{ Purpose...: Close opened file, free buffer
{============================================================================}
PROCEDURE File_CloseFile;

BEGIN

  IF (g_stFile <> '') THEN BEGIN
    {$I-}
    CLOSE(m_oFileTxt);
    {$I+}
    g_stFile := '';
  END;
END;
{====================================================== eoP File_CloseFile() }

{=========================================================== File_OpenFile() }
{ Purpose...: Open or create file, allocate buffer, if read mode, read file
{             into buffer
{ Input.....: i_bMode: m_bOPERATION_XXX
{ Result....: TRUE= sucess, FALSE= failed
{============================================================================}
FUNCTION File_OpenFile(i_bMode: BYTE): BOOLEAN;

  VAR fFnResult   : BOOLEAN;
  VAR bIoResult   : BYTE;
  VAR fReceive    : BOOLEAN;
  VAR fConfirm    : BOOLEAN;
  VAR fBufferFull : BOOLEAN;
  VAR stLine      : g_tStLine;
  VAR pstLine     : ^g_tStLine;
  VAR bLineLen    : BYTE;

BEGIN
  m_iBufUsed := 0;
  m_iBufPos := 0;
  m_iBufLine := 0;
  m_iBufLines := 0;

  fFnResult := TRUE;
  fConfirm := FALSE;
  fReceive := (i_bMode = g_bOPERATION_RECEIVE_KIM);
  WRITE(g_cCR);

  ASSIGN(m_oFileTxt, g_stFile);
  {$I-} RESET(m_oFileTxt) {$I+};
  bIoResult := IORESULT;

  {--------------------------------- read or view mode, check if file exists }
  IF ((fReceive = FALSE) AND (bIoResult <> 0)) THEN BEGIN
    WRITE('File ', g_stFile, ' not found.');
    fFnResult := FALSE;
    fConfirm := TRUE;
  END;

  {--------------------------- create mode, confirm overwrite if file exists }
  IF ((fReceive = TRUE) AND (bIoResult = 0)) THEN BEGIN
    WRITE(g_stFile, ' already exists. Overwrite? (Y/N) ');

    IF (UPCASE(Lib_GetKey('YNyn', TRUE)) = 'N') THEN BEGIN
      fFnResult := FALSE;
    END;
  END;

  {--------------------------------------------------- create/overwrite file }
  IF ((fFnResult = TRUE) AND (fReceive = TRUE)) THEN BEGIN
    {$I-} REWRITE(m_oFileTxt) {$I+};

    IF (IORESULT <> 0) THEN BEGIN
      WRITE('Error creating ', g_stFile);
      fFnResult := FALSE;
      fConfirm := TRUE;
    END;
  END;

  {--------------------------------------------------- init buffer pointers }
  IF (fFnResult = TRUE) THEN BEGIN
    m_pstBuf := PTR(ORD(m_pstBufStart));
    m_pcBuf := PTR(ORD(m_pstBufStart));
    m_pbBuf := PTR(ORD(m_pstBufStart));
    m_iBufLines := 0;
    m_iBufUsed := 0;
    m_iBufPos := 0;

    {---------------------------------------------------- fill buffer with 0 }
    FILLCHAR(m_pstBufStart^, m_iBufSize, 0);

    {--------------- if receive mode, print receiving prompt and buffer size }
    IF (fReceive = TRUE) THEN BEGIN
      CLRSCR;
      WRITE(g_stMSG_RECEIVE, g_stFile, ', ');
      WRITE(g_stMSG_BUFFER_SIZE);
      Lib_WriteInt(m_iBufSize, 0);
      WRITELN;
    END;

    {----------------------- if transmit or view mode, read file into buffer }
    IF (fReceive = FALSE) THEN BEGIN
      WRITE('Reading ', g_stFile, ' ... ');
      fBufferFull := FALSE;

      WHILE ((NOT EOF(m_oFileTxt)) AND (fBufferFull = FALSE)) DO BEGIN
        READLN(m_oFileTxt, stLine);
        bLineLen := LENGTH(stLine);
        m_iBufUsed := m_iBufUsed + bLineLen + 1;
        fBufferFull := (m_iBufUsed >= m_iBufSize);

        IF (fBufferFull = FALSE) THEN BEGIN
          pstLine := PTR(ORD(m_pstBuf));
          pstLine^ := stLine;
          m_pstBuf := PTR(ORD(m_pstBuf) + bLineLen + 1);
          m_iBufLines := m_iBufLines + 1;
        END ELSE BEGIN
          WRITE(g_cCR);
          CLREOL;
          WRITE(g_stMSG_BUFFER_FULL, 'at line ', m_iBufLines, '.');
          fConfirm := TRUE;
        END;
      END;
    END;
  END;

  {--------------------------------- some error shown, wait for confirmation }
  IF (fConfirm = TRUE) THEN BEGIN
    WRITE(' Any key to continue ');
    WHILE(NOT KEYPRESSED) DO BEGIN END;
    fFnResult := FALSE;
    File_CloseFile;
  END;

  File_OpenFile := fFnResult;
END;
{======================================================= eoF File_OpenFile() }

{======================================================== File_WriteBuffer() }
{ Purpose...: Write buffer to file
{============================================================================}
PROCEDURE File_WriteBuffer;

BEGIN

  IF (m_iBufUsed <> 0) THEN BEGIN
    WRITELN;
    WRITE('Writing buffer (');
    Lib_WriteInt(m_iBufUsed, 0);
    WRITE(' Bytes) to ', g_stFile, ' ... ');

    m_pcBuf := PTR(ORD(m_pstBufStart));

    WHILE (Lib_CmpInt(0, m_iBufUsed) <> g_bINT_EQ) DO BEGIN
      WRITE(m_oFileTxt, m_pcBuf^);
      m_pcBuf := PTR(ORD(m_pcBuf) + 1);
      m_iBufUsed := m_iBufUsed -1;
    END;
  END;

  File_CloseFile;
END;
{==================================================== eoF File_WriteBuffer() }

{========================================================== File_GetString() }
{ Purpose...: Read a string out of buffer
{ Input.....: i_iLine.: Starting line, -1 read next
{             i_fStart: TRUE= (re)start from beginning of buffer
{             i_pfEof.: Address where to store end-of-buffer status. It is
{                       TRUE if the current line is the last one.
{ Result....: Address of string, NIL if at end of buffer
{============================================================================}
FUNCTION File_GetString(i_iLine: INTEGER; i_fStart: BOOLEAN;
                        i_pfEof: g_tpBoolean): g_tpStLine;

  VAR pstBuf      : g_tpStLine;
  VAR pstFnResult : g_tpStLine;
  VAR fFound      : BOOLEAN;
  VAR bLineLen    : BYTE;
  VAR fEof        : BOOLEAN;

BEGIN
  pstFnResult := NIL;
  fFound := FALSE;

  {------------------------- set pointer to start if not read-next requested }
  IF (i_fStart = TRUE) THEN BEGIN
    m_pstBuf := m_pstBufStart;
    m_iBufLine := 0;
    m_iBufPos := 0;
  END;

  WHILE ((fFound = FALSE) AND (m_iBufPos < m_iBufUsed)) DO BEGIN

    IF (m_iBufLine >= i_iLine) THEN BEGIN
      pstFnResult := m_pstBuf;
      fFound := TRUE;
    END;

    bLineLen := LENGTH(m_pstBuf^);
    m_pstBuf := PTR(ORD(m_pstBuf) + bLineLen + 1);
    m_iBufPos := m_iBufPos + bLineLen + 1;
    m_iBufLine := m_iBufLine + 1;
  END;

  fEof := (m_iBufLine >= m_iBufLines);

  IF (i_pfEof <> NIL) THEN BEGIN
    i_pfEof^ := fEof;
  END;

  File_GetString := pstFnResult;
END;
{====================================================== eoP File_GetString() }

{============================================================ File_GetByte() }
{ Purpose...: Read a byte out of buffer
{ Input.....: i_pfEof.: Address where to store end-of-buffer status. It is
{                       TRUE if the current byte is the last one.
{ Result....: The read byte
{============================================================================}
FUNCTION File_GetByte(i_pfEof: g_tpBoolean): BYTE;

  VAR bFnResult : BYTE;
  VAR fEof      : BOOLEAN;

BEGIN
  bFnResult := 0;
  fEof := TRUE;

  IF (m_iBufPos <= m_iBufUsed) THEN BEGIN
    bFnResult := m_pbBuf^;
    m_pbBuf := PTR(ORD(m_pbBuf) + 1);
    m_iBufPos := m_iBufPos + 1;
    fEof := FALSE;
  END;

  IF (i_pfEof <> NIL) THEN BEGIN
    i_pfEof^ := fEof;
  END;

  File_GetByte := bFnResult;
END;
{======================================================== eoP File_GetByte() }

{========================================================== File_PutBuffer() }
{ Purpose...: Put a single byte into buffer
{ Input.....: i_cData: Character to write
{ Result....: TRUE= success, at least one more byte available
{             FALSE= out of buffer space
{============================================================================}
FUNCTION File_PutBuffer(i_cData: CHAR): BOOLEAN;

  VAR fFnResult : BOOLEAN;

BEGIN
  fFnResult := TRUE;

  IF (Lib_CmpInt(m_iBufSize, m_iBufUsed) = g_bINT_GT) THEN BEGIN

    IF (i_cData <> CHR(0)) THEN BEGIN
      m_pcBuf^ := i_cData;
      m_pcBuf := PTR(ORD(m_pcBuf) + 1);
      m_iBufUsed := m_iBufUsed + 1;
    END;

  END ELSE BEGIN
    WRITELN;
    WRITELN(g_stMSG_BUFFER_FULL);
    fFnResult := FALSE;
  END;

  File_PutBuffer := fFnResult;
END;
{====================================================== eoF File_PutBuffer() }

{======================================================== File_GetBufLines() }
{ Purpose...: Get number of lines in file buffer
{ Result....: Number of lines in buffer
{============================================================================}
FUNCTION File_GetBufLines: INTEGER;

BEGIN
  File_GetBufLines := m_iBufLines;
END;
{==================================================== eoF File_GetBufLines() }

{======================================================== File_GetBufBytes() }
{ Purpose...: Get number of bytes in file buffer
{ Result....: Number of bytes in buffer
{============================================================================}
FUNCTION File_GetBufBytes: INTEGER;

BEGIN
  File_GetBufBytes := m_iBufUsed;
END;
{==================================================== eoF File_GetBufBytes() }

{========================================================= File_GetBufSize() }
{ Purpose...: Get size of allocated buffer
{ Result....: Number of bytes in buffer
{============================================================================}
FUNCTION File_GetBufSize: INTEGER;

BEGIN
  File_GetBufSize := m_iBufSize;
END;
{===================================================== eoF File_GetBufSize() }

{=============================================================== File_Init() }
{ Purpose...: Initialize file buffer
{ Input.....: i?fInit: TRUE= allocate buffer, FALSE= free it
{ Output....: m_pstBufStart: Address of allocated buffer
{             m_iBufSize...: Size of allocated buffer
{============================================================================}
PROCEDURE File_Init(i_fInit: BOOLEAN);

BEGIN

  IF (i_fInit = TRUE) THEN BEGIN
    m_pstBufStart := NIL;
    m_iBufSize := MAXAVAIL;

    {---------------------------------------- eliminate negative buffer size }
    IF (m_iBufSize < 0) THEN BEGIN
      m_iBufSize := 32767;
    END;

    GETMEM(m_pstBufStart, m_iBufSize);
  END

  ELSE BEGIN
    IF (m_pstBufStart <> NIL) THEN BEGIN
      FREEMEM(m_pstBufStart, m_iBufSize);
      m_pstBufStart := NIL;
    END;
  END;
END;
{=========================================================== eoP Pile_Init() }

{########################################################## EOF KIM1FILE.INC }
