{############################################################## KIM1MENU.INC }
{ Purpose...: KIM1 menu and file selection
{ DependsOn.: LIB.INC, LIBPRT.INC, KIM1FILE.INC
{ Compiler..: Turbo-Pascal V3.0
{ Target....: Kaypro II
{ LastEdit..: 24.01.2023
{############################################################################}

{----- line number where selection prompt is printed (used for file listing) }
CONST m_bPROMPT_LINE = 13;

{----------------------------------------------- length of menu border lines }
CONST m_bBORDER_LENGTH = 25;

{************************************************************ implementation }
{========================================================= Menu_SelectFile() }
{ Purpose...: Select a file
{ Input.....: i_bOperation: m_bOPERATION_XXX
{ Result....: TRUE= File entered/selected, FALSE= aborted
{============================================================================}
FUNCTION Menu_SelectFile(i_bOperation: BYTE): BOOLEAN;

  CONST bMODE_TRANSMIT = 1;
  CONST bMODE_RECEIVE  = 2;
  CONST bMODE_VIEW     = 3;
  CONST bMODE_PRINT    = 4;

  VAR stFileExt: g_tStFile;
  VAR bCnt:      BYTE;
  VAR bMode:     BYTE;

BEGIN
  bMode := bMODE_RECEIVE;
  stFileExt := '';
  WRITE(g_cCR, 'Reading directory...');

  CASE i_bOperation OF
    g_bOPERATION_RECEIVE_KIM:  BEGIN
                                 bMode := bMODE_RECEIVE;
                                 stFileExt := '.KIM';
                               END;
    g_bOPERATION_TRANSMIT_KIM: BEGIN
                                 bMode := bMODE_TRANSMIT;
                                 stFileExt := '.KIM';
                               END;
    g_bOPERATION_VIEW_TXT:     BEGIN
                                 bMode := bMODE_VIEW;
                                 stFileExt := '.???';
                               END;
    g_bOPERATION_PRINT_TXT:    BEGIN
                                 bMode := bMODE_PRINT;
                                 stFileExt := '.???';
                               END;
  END;

  {------------------------------- display directory of selected file types }
  WRITELN; WRITELN;
  IF (stFileExt = '.KIM') THEN WRITE('KIM');
  IF (stFileExt = '.???') THEN WRITE('Text');
  WRITELN(' files on disk');
  Lib_PrintDirectory('????????' + COPY(stFileExt,2,4));

  {----------------------------------------------------------- get filename }
  GOTOXY(1, m_bPROMPT_LINE);
  CLREOL;

  IF (stFileExt <> '.???') THEN BEGIN
    WRITE(g_cCR, 'File (', stFileExt, ') to ');
  END ELSE BEGIN
    WRITE(g_cCR, 'File to ');
  END;

  CASE bMode OF
    bMODE_TRANSMIT: WRITE('transmit ');
    bMODE_RECEIVE:  WRITE('receive ');
    bMODE_VIEW:     WRITE('view ');
    bMODE_PRINT:    WRITE('print ');
  END;

  g_stFile := '';
  READ(g_stFile);
  WRITE(g_cCR);

  {-------------------------------- clear area occupied by directory listing }
  FOR bCnt := m_bPROMPT_LINE TO 23 DO BEGIN
    GOTOXY(1, bCnt);
    CLREOL;
  END;

  GOTOXY(1, m_bPROMPT_LINE);

  IF (stFileExt <> '.???') THEN BEGIN
    IF (LENGTH(g_stFile) >8) THEN BEGIN
      g_stFile := COPY(g_stFile, 1, 8);
    END;
  END;

  {-------------------------------------------------------- open/create file }
  IF (LENGTH(g_stFile) <> 0) THEN BEGIN

    IF (stFileExt <> '.???') THEN BEGIN
      g_stFile := Lib_CnvUpCase(g_stFile + stFileExt);
    END ELSE BEGIN
      g_stFile := Lib_CnvUpCase(g_stFile);
    END;

    IF (bMode <> bMODE_PRINT) THEN BEGIN
      IF (File_OpenFile(i_bOperation) = FALSE) THEN BEGIN
        g_stFile := '';
      END;
    END;
  END;

  Menu_SelectFile := (LENGTH(g_stFile) <> 0);
END;
{===================================================== eoF Menu_SelectFile() }

{========================================================== Menu_PrintFile() }
{ Purpose...: Select a print mode, send command to printer
{============================================================================}
PROCEDURE Menu_PrintFile;

  VAR bCnt : BYTE;

BEGIN
  WRITE('Printing ' + g_stFile);
  LibPrt_PrintFile(g_stFile, 1, m_bPROMPT_LINE + 1);

  {----------------------------------------- clear area occupied by messages }
  FOR bCnt := m_bPROMPT_LINE TO (m_bPROMPT_LINE + 2) DO BEGIN
    GOTOXY(1, bCnt);
    CLREOL;
  END;

  GOTOXY(1, m_bPROMPT_LINE);
END;
{====================================================== eoF Menu_PrintFile() }

{==================================================== Menu_SelectPrintMode() }
{ Purpose...: Select a print mode, send command to printer
{============================================================================}
PROCEDURE Menu_SelectPrintMode;

  TYPE tstMode    = STRING[10];

  VAR cKbd         : CHAR;
  VAR fDone        : BOOLEAN;
  VAR fDummy       : BOOLEAN;
  VAR stMode       : tstMode;
  VAR iLeftMargin  : INTEGER;
  VAR piLeftMargin : ^INTEGER;
  VAR bPageLength  : BYTE;
  VAR bPageLines   : BYTE;
  VAR iPageNum     : INTEGER;

BEGIN
  fDone := FALSE;
  stMode := '';
  piLeftMargin := PTR(ADDR(iLeftMargin));

  {---------------------------------------------------------------- get mode }
  WHILE (fDone = FALSE) DO BEGIN
    GOTOXY(1, m_bPROMPT_LINE);
    CLREOL;
    WRITE('Init  Pica  Elite  Cond  eMph  mArgin  Form  pageNbr ');
    cKbd := UPCASE(Lib_GetKey('IiPpCcEeMmAaFfNn' + g_cESC + g_cCR, TRUE));
    GOTOXY(1, m_bPROMPT_LINE);
    CLREOL;

    LibPrt_Setup(1, m_bPROMPT_LINE, FALSE, -1);

    CASE cKbd OF
      'I'  : stMode := g_cLibPrtMODE_INIT;
      'P'  : stMode := g_cLibPrtMODE_PICA;
      'E'  : stMode := g_cLibPrtMODE_ELITE;
      'C'  : stMode := g_cLibPrtMODE_COND;
      'M'  : stMode := g_cLibPrtMODE_EMPH;
      'A'  : stMode := g_cLibPrtMODE_LMARGIN;
    END;

    fDone := ((cKbd = g_cESC) OR (cKbd = g_cCR));

    IF (fDone = FALSE) THEN BEGIN

      {------------------------------------------- enter left printer margin }
      IF (cKbd = 'A') THEN BEGIN
        WRITE('Left margin: ');

        IF (Lib_GetNum(piLeftMargin, NIL, 3, FALSE) = TRUE) THEN BEGIN
          stMode := stMode + CHR(iLeftMargin);
        END ELSE BEGIN
          stMode := '';
        END;

        GOTOXY(1, m_bPROMPT_LINE);
        CLREOL;
      END;

      {------------------------------------------------- print page numbers }
      IF (cKbd = 'N') THEN BEGIN
        WRITE('Print page numbers (Y/N): ');
        cKbd := UPCASE(Lib_GetKey('YyNn' + g_cESC + g_cCR, TRUE));
        iPageNum := 0;
        IF (cKbd = 'Y') THEN iPageNum := 1;
        LibPrt_Setup(-1, -1, FALSE, iPageNum);
        stMode := '';

        GOTOXY(1, m_bPROMPT_LINE);
        CLREOL;
      END;

      {------------------------------------------------- select paper format }
      IF (cKbd = 'F') THEN BEGIN
        WRITE('<A>4  <B>5  <C>ontinuous ');
        cKbd := UPCASE(Lib_GetKey('AaBbCc' + g_cESC + g_cCR, TRUE));
        bPageLength := 0;
        bPageLines := 0;

        IF (cKbd = 'A') THEN BEGIN
          bPageLength := g_bLibPrtPAGE_LENGTH_A4;
          bPageLines := g_bLibPrtPAGE_LINES_A4;
        END;

        IF (cKbd = 'B') THEN BEGIN
          bPageLength := g_bLibPrtPAGE_LENGTH_B5;
          bPageLines := g_bLibPrtPAGE_LINES_B5;
        END;

        IF (cKbd = 'C') THEN BEGIN
          bPageLength := g_bLibPrtPAGE_LENGTH_CONT OR $80;
          bPageLines := g_bLibPrtPAGE_LINES_CONT;
        END;

        IF (bPageLength <> 0) THEN BEGIN
          LibPrt_SetForm(bPageLength, bPageLines);
        END;
      END;

      IF (LENGTH(stMode) <> 0) THEN BEGIN
        fDummy := LibPrt_Print(CHR(0), g_cESC + stMode);
      END;
    END;
  END;
END;
{================================================ eoP Menu_SelectPrintMode() }

{============================================================= Menu_Border() }
{ Purpose...: Draw menu borders. Implemented to save space (hopefully)
{============================================================================}
PROCEDURE Menu_Border;

  VAR bCnt : BYTE;

BEGIN
  FOR bCnt := 0 TO m_bBORDER_LENGTH DO BEGIN
    WRITE('=');
  END;

  WRITELN;
END;
{=========================================================== eoP Menu_Border }

{==================================================================== Menu() }
{ Purpose...: Display menu
{ Input.....: i_piSelection: Where to store selection
{ Result....: TRUE= a selection was made, FALSE= exit was selected
{============================================================================}
FUNCTION Menu(i_piOperation: INTEGER): BOOLEAN;

  VAR cKbd       : CHAR;
  VAR bOperation : BYTE;
  VAR fDrawMenu  : BOOLEAN;
  VAR fSelectFile: BOOLEAN;
  VAR stFileExt  : g_tStFile;

BEGIN
  fDrawMenu := TRUE;
  bOperation := 0;

  WHILE (bOperation = 0) DO BEGIN

    IF (fDrawMenu = TRUE) THEN BEGIN
      fDrawMenu := FALSE;
      CLRSCR;
      Menu_Border;
      WRITELN(g_stPROGRAM, ' V', g_stVERSION);
      Menu_Border;

      WRITE('-1- Receive KIM file'     + g_stCRLF +
            '-2- Transmit KIM file'    + g_stCRLF +
            '-T- Terminal'             + g_stCRLF +
            '-V- View text file'       + g_stCRLF +
            '-R- Print text file'      + g_stCRLF +
            '-U- Usage notes'          + g_stCRLF +
            '-P- Setup printer'        + g_stCRLF +
            '-X- Exit'                 + g_stCRLF
           );

      Menu_Border;
    END;

    WRITE(g_cCR);
    CLREOL;
    WRITE('Selection ');
    cKbd := UPCASE(Lib_GetKey('12TtVvRrUuPpXx', TRUE));
    fSelectFile := TRUE;

    CASE cKbd OF

      '1': bOperation := g_bOPERATION_RECEIVE_KIM;
      '2': bOperation := g_bOPERATION_TRANSMIT_KIM;

      'T': BEGIN
             bOperation := g_bOPERATION_TERMINAL;
             fSelectFile := FALSE;
           END;

      'V': bOperation := g_bOPERATION_VIEW_TXT;
      'R': bOperation := g_bOPERATION_PRINT_TXT;

      'U': BEGIN
             bOperation := g_bOPERATION_USAGE;
             fSelectFile := FALSE;
             g_stFile := g_stFILE_USAGE;

             IF (File_OpenFile(g_bOPERATION_VIEW_TXT) = FALSE) THEN BEGIN
               bOperation := 0;
               g_stFile := '';
             END;
           END;

      'P': BEGIN
             bOperation := 0;
             fSelectFile := FALSE;
             Menu_SelectPrintMode;
           END;

      'X': BEGIN
             bOperation := g_bOPERATION_EXIT;
             fSelectFile := FALSE;
           END;
    END;

    {---------------------------------------------- select file to work with }
    IF (fSelectFile = TRUE) THEN BEGIN

      IF (Menu_SelectFile(bOperation) = FALSE) THEN BEGIN
        bOperation := 0;
      END;

      IF (bOperation = g_bOPERATION_PRINT_TXT) THEN BEGIN
        bOperation := 0;
        Menu_PrintFile;
      END;
    END;
  END;

  MEM[i_piOperation] := bOperation;
  Menu := (bOperation <> g_bOPERATION_EXIT);
END;
{================================================================ eoF Menu() }

{########################################################## EOF KIM1MENU.INC }
